/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "CvfdImage.h"

using namespace std;
using namespace FD;
      


CvfdImage::CvfdImage()
  : m_image(NULL)
{
}
   
CvfdImage::CvfdImage(const IplImage* image)
{
  if(image != NULL)
    {
      int status = cvGetErrMode();
      cvSetErrMode( CV_ErrModeSilent ); 
      __BEGIN__;
      OPENCV_CALL(m_image = cvCloneImage(image));
      __END__;
      cvSetErrMode( status );
      if( cvGetErrStatus() != CV_StsOk  )
	{
	  throw new GeneralException("OPENCV - Error to clone the image " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
	}
    }
  else
    {
      m_image = 0;
    }
} 
   
CvfdImage::CvfdImage(const CvfdImage* image)
{
  int status = cvGetErrMode();
  cvSetErrMode( CV_ErrModeSilent ); 
  __BEGIN__;
  OPENCV_CALL(m_image = cvCloneImage(image->getImage()));
  __END__;
  cvSetErrMode( status );
  if( cvGetErrStatus() != CV_StsOk  )
    {
      throw new GeneralException("OPENCV - Error to clone the image " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
    }
}
   
CvfdImage::CvfdImage(CvSize size, int depth, int channels)
{
  int status = cvGetErrMode();
  cvSetErrMode( CV_ErrModeSilent );
  __BEGIN__;         
  OPENCV_CALL(m_image = cvCreateImage( size, depth, channels));
  OPENCV_CALL(cvSetZero(m_image));
  __END__;
  cvSetErrMode( status );
  if( cvGetErrStatus() != CV_StsOk  )
    {            
      throw new GeneralException("OPENCV - Error to create an image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
    }          
}
   
CvfdImage::~CvfdImage()
{               
  if(m_image != NULL)
    {
                 
      cvReleaseImage( &m_image );
    }
}
	
void CvfdImage::releaseImage()
{                
  if(m_image != NULL)
    {  
      cout << "cvReleaseImage( &m_image )"<<endl;        
      cvReleaseImage( &m_image );
      m_image = NULL;
    }
}
   
CvfdImage::CvfdImage(const std::string &path, const int &iscolor)
{
  int status = cvGetErrMode();
  cvSetErrMode( CV_ErrModeSilent );
  __BEGIN__;
  OPENCV_CALL(m_image = cvLoadImage(path.c_str(), iscolor));
  __END__;
  cvSetErrMode( status );
  if( cvGetErrStatus() != CV_StsOk  )
    {
      throw new GeneralException("OPENCV - Error to load the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
    }
  if(m_image == NULL)
    {
      throw new GeneralException("OPENCV - Error to load the image: it's NULL check your path" ,__FILE__,__LINE__);
    }
}
   
/**Serialize (binary) the object to a stream*/
void CvfdImage::serialize(std::ostream &out) const
{
      
}
   
/**Unserialize (binary) the object from a stream*/
void CvfdImage::unserialize(std::istream &in)
{
      
}
   
/**Generic print function*/
void CvfdImage::printOn(std::ostream &out) const
{
  out << "CvfdImage: ";
  out << m_image;
}
   
/**Generic read function*/
void CvfdImage::readFrom(std::istream &in)
{
      
}
   
IplImage* CvfdImage::getImage() const
{
  return m_image;
}
   
void CvfdImage::convertImage(CvSize size, int depth, int channels)
{
  CvfdImage* image = NULL;
  int status = cvGetErrMode();
  cvSetErrMode( CV_ErrModeSilent );
  __BEGIN__;         
  OPENCV_CALL(image = new CvfdImage(size, depth, channels));
  OPENCV_CALL(cvConvertImage(m_image, image->getImage()));
  OPENCV_CALL(m_image = cvCloneImage(image->getImage()));
  __END__;
  cvSetErrMode( status );
  if( cvGetErrStatus() != CV_StsOk  )
    {            
      throw new GeneralException("OPENCV - Error to convert an image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
    }          
}

CvfdImage* CvfdImage::gray() const
{
  CvfdImage* image = NULL;
  if(m_image->nChannels != 1)
    {     
      int status = cvGetErrMode();
      cvSetErrMode( CV_ErrModeSilent );
      __BEGIN__;         
      OPENCV_CALL(image = new CvfdImage(cvGetSize(m_image), m_image->depth, 1));
      OPENCV_CALL(cvCvtColor(m_image, image->getImage(), CV_BGR2GRAY));
      __END__;
      cvSetErrMode( status );
      if( cvGetErrStatus() != CV_StsOk  )
	{            
	  throw new GeneralException("OPENCV - Error to convert in gray the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
	}           
    }
  else
    {
      image = new CvfdImage(m_image);
    }    
  return image;
}   
   
CvfdImage* CvfdImage::bgr() const
{
  CvfdImage* image;    
  if(m_image->nChannels == 1)
    {
      int status = cvGetErrMode();
      cvSetErrMode( CV_ErrModeSilent );
      __BEGIN__; 
      OPENCV_CALL(image = new CvfdImage(cvGetSize(m_image), m_image->depth, 3));
      OPENCV_CALL(cvCvtColor(m_image, image->getImage(), CV_GRAY2BGR));
      __END__;
      cvSetErrMode( status );
      if( cvGetErrStatus() != CV_StsOk  )
	{
	  throw new GeneralException("OPENCV - Error to convert in rgb the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
	} 
    }
  else
    {
      image = new CvfdImage(m_image);
    }
  return image;
} 
   
CvfdImage* CvfdImage::zero() const
{
  CvfdImage* image = new CvfdImage(m_image);      
  int status = cvGetErrMode();
  cvSetErrMode( CV_ErrModeSilent );
  __BEGIN__;       
  OPENCV_CALL(cvSetZero(image->getImage()));
  __END__;      
  cvSetErrMode( status );
  if( cvGetErrStatus() != CV_StsOk  )
    {
      throw new GeneralException("OPENCV - Error to clear the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
    }       
  return image;
}
   
