/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;

class CvfdhSub;
   
DECLARE_NODE(CvfdhSub)
/*Node
 *
 * @name CvfdhSub
 * @category FDOPENCV_HARK:Arithmetic
 * @description CvfdhSub
 *
 * @input_name IMAGEIN1
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @input_name IMAGEIN2
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @output_name IMAGEOUT
 * @output_description Output image
 * @output_type CvfdImage
 *
 END*/
   
   
class CvfdhSub : public BufferedNode {

  int m_imageIn1ID;
  int m_imageIn2ID;
  int m_imageOutID;
      
public:
  CvfdhSub(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageIn1ID = addInput("IMAGEIN1"); 
    m_imageIn2ID = addInput("IMAGEIN2");
    m_imageOutID = addOutput("IMAGEOUT");    
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    RCPtr<CvfdImage> imagePtr1 = getInput(m_imageIn1ID,count);   
    if(imagePtr1->getImage() == NULL){
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr1);
      return;
    }

    RCPtr<CvfdImage> imagePtr2 = getInput(m_imageIn2ID,count);   
    if(imagePtr2->getImage() == NULL){
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr2);
      return;
    }

    IplImage* img_in1;
    img_in1 = cvCloneImage(imagePtr1->getImage());

    IplImage* img_in2;
    img_in2 = cvCloneImage(imagePtr2->getImage());
    
    // ---------------------------------------------------------------
    // main routine : start

    IplImage *dst_img = 0;
    
    dst_img = cvCloneImage (img_in1);
    
    cvSub(img_in1, img_in2, dst_img);
    
    IplImage* img_out;
    img_out = cvCloneImage(dst_img);

    // main routine : finish
    // ---------------------------------------------------------------
         
    CvfdImage* image= new CvfdImage(img_out);  
         
    (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

    cvReleaseImage(&img_in1);
    cvReleaseImage(&img_in2);
    cvReleaseImage(&dst_img);
    cvReleaseImage(&img_out);

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhSub)
};
